/******************************************************************************

Licensed Materials - Property of IBM 

5655-S97 

© Copyright IBM Corp. 2012 

This source material is provided "AS-IS" under the terms and
conditions of the IBM Customer Agreement and of the associated
Licensed Program Specifications documentation for CICS Transaction
Server for z/OS.

The terms and conditions of this license permit users to modify this
source material and DO NOT provide for any entitlement to defect
correction.

******************************************************************************/
package  $packageName$;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.jface.text.source.VerticalRuler;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.events.HyperlinkAdapter;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.part.ViewPart;

import com.ibm.cics.core.comm.ConnectionException;
import com.ibm.cics.core.comm.IConnectionCategory;
import com.ibm.cics.core.connections.ConnectionServiceListener;
import com.ibm.cics.core.connections.ConnectionsPlugin;

public class TelnetCommandViewPart extends ViewPart {

	private ConnectionServiceListener connectionServiceListener;
	private ITelnetConnection telnetConnection;
	private StackLayout stackLayout;
	private Composite noConnectionCompositie;
	private SourceViewer sourceViewer;
	private Thread inputJob;
	private Text commandText;
	private Composite connectedComposite;
	private Composite stackComposite;
	
	private class MyConnectionServiceListener extends ConnectionServiceListener {
		@Override
		public void event(ConnectionServiceEvent event) {
			if(event.getConnectionCategoryId().equals(TelnetConnection.CATEGORY)){
				if(event instanceof ConnectedEvent){						
					setTelnetConnection((ITelnetConnection) event.getConnectable().getConnection());
				} else if(event instanceof DisconnectedEvent){
					setTelnetConnection(null);
				}
			}
		}
	}

	public TelnetCommandViewPart() {
	}

	@Override
	public void createPartControl(Composite parent) {
		
		FormToolkit toolkit = new FormToolkit(parent.getDisplay());
						
		stackComposite = toolkit.createComposite(parent,SWT.INHERIT_FORCE);
		stackLayout = new StackLayout();
		stackComposite.setLayout(stackLayout);
		
		noConnectionCompositie = toolkit.createComposite(stackComposite,SWT.INHERIT_FORCE);
		noConnectionCompositie.setBackground(parent.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		noConnectionCompositie.setLayout(new GridLayout(1,false));
		toolkit.createLabel(noConnectionCompositie,"Only available with an active Telnet Connection");
		
		// Create a hyperlink to create a telnet connection
		Hyperlink createConnectionHyperlink = toolkit.createHyperlink(noConnectionCompositie, "Create Telnet Connection", SWT.TRANSPARENT);
		createConnectionHyperlink.addHyperlinkListener(new HyperlinkAdapter(){
			@Override
			public void linkActivated(HyperlinkEvent e) {
				// Drive the UI to the screen to add a new telnet connection
				ConnectionsPlugin.getDefault().getConnectionUserInterfaceService().createConnectionConfiguration(TelnetConnection.ID);
			}
		});
		
		createSourceViewer(stackComposite);
		
		layoutStackPanes();
						
		stackComposite.layout();
		
		ConnectionsPlugin.getDefault().getConnectionService().addConnectionServiceListener(new MyConnectionServiceListener());
		
	}
	
	private void createSourceViewer(Composite aParent){
		
		connectedComposite = new Composite(aParent,SWT.NONE);
		
		GridLayout gridLayout = new GridLayout(1,false);
		gridLayout.marginHeight = gridLayout.marginWidth = 0;
		gridLayout.verticalSpacing = gridLayout.marginTop = 2;
		connectedComposite.setLayout(gridLayout);		
		
		Composite topComposite = new Composite(connectedComposite,SWT.NONE);
		GridData data = new GridData(SWT.FILL,SWT.NONE,true,false);
		topComposite.setLayoutData(data);
		
		gridLayout = new GridLayout(2,false);
		gridLayout.marginHeight = 0 ;
		gridLayout.verticalSpacing = gridLayout.marginTop = 2;
		topComposite.setLayout(gridLayout);
		
		Label l =  new Label(topComposite,SWT.NONE);
		l.setText("Command:");
		
		commandText = new Text(topComposite,SWT.BORDER);
		data = new GridData(SWT.FILL,SWT.NONE,true,false);
		commandText.setLayoutData(data);
		commandText.addSelectionListener(new SelectionAdapter(){
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				runCommand(commandText.getText());
				commandText.setText("");
			}
		});
		
		VerticalRuler verticalRuler = new VerticalRuler(1);
		int styles= SWT.V_SCROLL | SWT.H_SCROLL | SWT.MULTI | SWT.BORDER | SWT.FULL_SELECTION;
		
		sourceViewer = new SourceViewer(connectedComposite, verticalRuler, styles);
		data = new GridData(SWT.FILL,SWT.FILL,true,true);
		sourceViewer.getControl().setLayoutData(data);
		
		sourceViewer.setEditable(false);
		Font blockFont= JFaceResources.getFont("org.eclipse.jface.textfont"); //$NON-NLS-1$
		sourceViewer.getTextWidget().setFont(blockFont);
		verticalRuler.setFont(blockFont);
		
	}

	@Override
	public void setFocus() {
		commandText.setFocus();
	}
	
	private void runCommand(String aString){
		try {
			commandText.setEnabled(false);
			sourceViewer.setDocument(new Document("Executing..." + aString));			
			write(telnetConnection.getOperation().execute(aString));
			commandText.setEnabled(true);
			commandText.setFocus();
		} catch (ConnectionException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
		ConnectionsPlugin.getDefault().getConnectionService().addConnectionServiceListener(new MyConnectionServiceListener());
	}
	
	@Override
	public void dispose() {
		super.dispose();
		if (connectionServiceListener != null) connectionServiceListener.makeStale();
	}
	
	private void setTelnetConnection(ITelnetConnection connection) {
		telnetConnection = connection;
		Runnable runnable = new Runnable(){
			public void run(){
				if(stackLayout !=null){
					layoutStackPanes();
				}			
			}
		};
		if(Display.getCurrent() == null){
			stackComposite.getDisplay().asyncExec(runnable);
		} else {
			runnable.run();
		}
	}
	
	private void layoutStackPanes(){
		if(telnetConnection == null){
			stackLayout.topControl = noConnectionCompositie;
			stackComposite.layout();
			if(inputJob != null){
				inputJob.interrupt();
				inputJob = null;
			}
		} else {
			stackLayout.topControl = connectedComposite;
			stackComposite.layout();	
			write(telnetConnection.getInitialMessage());
		}
	}
	
	private void write(final String aString){
		sourceViewer.getControl().getDisplay().asyncExec(new Runnable(){
			public void run(){
				sourceViewer.setDocument(new Document(aString));				
			}
		});
	}
	
	public String getPartProperty(String key) {
		if(IConnectionCategory.class.getName().equals(key)){
			return TelnetConnection.CATEGORY;
		} else {
			return super.getPartProperty(key);
		}
	}

}
