/******************************************************************************

Licensed Materials - Property of IBM 

5655-S97 

© Copyright IBM Corp. 2012 

This source material is provided "AS-IS" under the terms and
conditions of the IBM Customer Agreement and of the associated
Licensed Program Specifications documentation for CICS Transaction
Server for z/OS.

The terms and conditions of this license permit users to modify this
source material and DO NOT provide for any entitlement to defect
correction.

******************************************************************************/
package $packageName$;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.SocketException;
import java.text.MessageFormat;

import javax.security.sasl.AuthenticationException;

import org.apache.commons.net.telnet.TelnetClient;

import com.ibm.cics.core.comm.AbstractConnection;
import com.ibm.cics.core.comm.ConnectionException;

public class TelnetConnection extends AbstractConnection implements ITelnetConnection {
	
	public static final String ID = "com.ibm.cics.zos.examples.telnet.connections";
	public static final String CATEGORY = "com.ibm.cics.zos.examples.telnet.category";
	public static final String TIMEOUT_KEY = "TIMEOUT";
	
	private TelnetClient tc;
	
	private TelnetReadThread inputRunnable;
	private String initialMessage;
	
	private static String END_IMMEDIATE = "END_IMMEDIATE";

	public TelnetConnection() {
	}
	
	@Override
	public String getInitialMessage(){
		return initialMessage;
	}

	@Override
	public void connect() throws ConnectionException {
		
		tc = getNewTelnetClient();
        	      
	}
	
	private void setCustomizerValues(TelnetClient telnetConnection) throws ConnectionException{
		
		// This value is to show how a customizer can be used to get extra details that can be used at connect time
		String connectionTimeoutString = getConfiguration().getExtendedAttribute(TIMEOUT_KEY);
		if(connectionTimeoutString != null && connectionTimeoutString.length() > 0){
			try{
				telnetConnection.setConnectTimeout(Integer.valueOf(connectionTimeoutString));
			} catch (NumberFormatException e){
				throw new ConnectionException(MessageFormat.format("Timeout value {0} must be an integer", connectionTimeoutString), e);
			}
		}
		
	}
	
	@SuppressWarnings({ "deprecation", "unused" })
	private TelnetClient getNewTelnetClient() throws ConnectionException{
		TelnetClient result = new TelnetClient();
		
		setCustomizerValues(result);
		
		try{
			result.connect(getHost(), getPort());
			String logonMessageforUserID = readInputStream(result,"EZYTE27I",":");
			writeOutputStream(result,getUserID());			
			String logonMessageForPassword = readInputStream(result,"EZYTE28I",":");
			writeOutputStream(result,getPassword());
			initialMessage = readInputStream(result, END_IMMEDIATE);
			if(initialMessage.trim().startsWith("EZYTE32W")){
				throw new AuthenticationException(initialMessage.trim());
			} else {					
				return result;
			}
		} catch (SocketException e) {
			throw new ConnectionException(e);
		} catch (IOException e) {
			throw new ConnectionException(e);
		}
	}
	
	private void writeOutputStream(TelnetClient aTelnetClient, String aString){
		try{
			startInputRunnable(aTelnetClient);
			System.out.println("WRITE->" + aString);
			aTelnetClient.getOutputStream().write(aString.getBytes());
			aTelnetClient.getOutputStream().write(System.getProperty("line.separator").getBytes());
			aTelnetClient.getOutputStream().flush();
		} catch (IOException e){
			e.printStackTrace();
		}
	}
	
	private void startInputRunnable(TelnetClient aTelnetClient){
		if(inputRunnable == null){
			inputRunnable = new TelnetReadThread(aTelnetClient);	
			inputRunnable.start();
		}
	}
	
	private String readInputStream(TelnetClient aTelnetClient, final String upToMessage){
		
		if(upToMessage == END_IMMEDIATE){
 			return readInputStream(aTelnetClient, upToMessage, null);
 		} else {
 			return readInputStream(aTelnetClient, null, upToMessage == null ? ":>" : upToMessage);
 		}
		
	}
	
	private String readInputStream(TelnetClient aTelnetClient, final String upToMessage, final String upToMessageDelimiter){		
		try {
			if(inputRunnable == null){
				inputRunnable = new TelnetReadThread(aTelnetClient,upToMessage,upToMessageDelimiter);
				inputRunnable.start();
			} else {
				inputRunnable.setEndInputMessage(upToMessage,upToMessageDelimiter);
			}
			while(inputRunnable.isAlive()){
				Thread.sleep(200);
				// If the thread is alive it is possible it has jammed reading the input stream
				boolean testEndCondition = inputRunnable.testEndCondition();
				if(testEndCondition){
					inputRunnable.interrupt();
					break;
				} else {
					Thread.sleep(200);
				}
			}
			String result = inputRunnable.inputResult;
			inputRunnable = null;
			return result;
		} catch (InterruptedException e) {
			e.printStackTrace();
			return null;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
	
	public TelnetClient getTelnetClient(){
		return tc;
	}
	
	public InputStream getInputStream(){
		return tc.getInputStream();
	}
	
	@Override
	public OutputStream getOutputStream() {
		return tc.getOutputStream();
	}

	@Override
	public void disconnect() throws ConnectionException {
		if(tc != null){
			try {
				tc.disconnect();
			} catch (Exception e) {
				// We do not care if disconnect fails - which it seems to do quite easily with an NPE
				// Just catch it and carry on
			} finally {
				tc = null;				
			}
		}
	}

	@Override
	public boolean isConnected() {
		return tc != null && tc.isConnected();
	}

	@Override
	public TelnetOperation getOperation() {
		return new TelnetOperation(this);
	}
	
	public String runExecutableFile(String path, String aFileName) throws ConnectionException {

		TelnetClient telnetClient = getNewTelnetClient();
		writeOutputStream(telnetClient, "cd " + path);
		readInputStream(telnetClient, ":>");
		writeOutputStream(telnetClient, "./" + aFileName);
		return readInputStream(telnetClient, ":>");
		
	}


	public String execute(String aCommand) throws ConnectionException {

//		TelnetClient telnetClient = getNewTelnetClient();
		writeOutputStream(tc, aCommand);
		return readInputStream(tc, ":>");
		
	}

}
