/******************************************************************************

Licensed Materials - Property of IBM 

5655-S97 

© Copyright IBM Corp. 2012 

This source material is provided "AS-IS" under the terms and
conditions of the IBM Customer Agreement and of the associated
Licensed Program Specifications documentation for CICS Transaction
Server for z/OS.

The terms and conditions of this license permit users to modify this
source material and DO NOT provide for any entitlement to defect
correction.

******************************************************************************/
package $packageName$;

import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.net.telnet.TelnetClient;

public class TelnetReadThread extends Thread {
	
	String endInputMessage;
	String inputResult;
	private TelnetClient telnetClient; 
	public static String END_IMMEDIATE = "END_IMMEDIATE";
	String endInputMessageDelimiter = ":>";
	private StringBuffer buffer = new StringBuffer();
	private int ret_read;
	
	public TelnetReadThread(TelnetClient aTelnetClient){
		telnetClient = aTelnetClient;
	}
	
	public TelnetReadThread(TelnetClient aTelnetClient, String upToMessage){
		this(aTelnetClient);
		endInputMessage = upToMessage;
	}
	
	public TelnetReadThread(TelnetClient aTelnetClient, String upToMessage, String upToMessageDelimiter) {
		this(aTelnetClient,upToMessage);
		endInputMessageDelimiter = upToMessageDelimiter;
	}
	
	@Override
	public void interrupt() {
		super.interrupt();
	}

	@Override
	public void run() {
		
		InputStream inputStream = telnetClient.getInputStream();
		try {
			byte[] buff = new byte[1024];
			ret_read = 0;
			do {
				ret_read = 0;
				ret_read = inputStream.read(buff);
	            if(ret_read > 0) {
	            	String inputString = new String(buff, 0, ret_read);
	            	if(inputString.trim().length() == 0){
	            		inputString = inputString.trim();
	            	}
	            	buffer.append(inputString);	            	
	            }
	            // If we were asked to look for a particular message keep reading the buffer until we have this message
	            if(testEndCondition()){
	            	break;
	            }
	        } while (ret_read >= 0);
	    } catch (IOException e)  {
	    	System.err.println("Exception while reading socket:" + e.getMessage());
	    }			
	}

	public void setEndInputMessage(String aString, String upToMessageDelimiter) {
		endInputMessage = aString;
		endInputMessageDelimiter = upToMessageDelimiter;
	}

	public void setEndInputMessageDelimiter(String upToMessageDelimiter) {
		endInputMessageDelimiter = upToMessageDelimiter;
	}

	public boolean testEndCondition() {
		if(ret_read >0){
			return false; //We are still getting input off the stream to don't process the result yet until the stream has been exhausted
		}
        if(endInputMessage == END_IMMEDIATE){
        	//Do not return if nothing has come back yet
        	if(buffer.toString().trim().length() == 0){
        		return false;
        	}
        	inputResult = buffer.toString();
        	System.out.println("READ> " + inputResult);
        	return true;
        } else if (endInputMessage != null && endInputMessageDelimiter != null){
        	if(buffer.toString().trim().startsWith(endInputMessage) && buffer.toString().trim().endsWith(endInputMessageDelimiter)){
            	inputResult = buffer.toString();
            	System.out.println("READ> " + inputResult);
            	return true;
        	}
        } else if(endInputMessageDelimiter != null && buffer.toString().trim().endsWith(endInputMessageDelimiter)){
        	inputResult = buffer.toString();
        	System.out.println("READ> " + inputResult);
        	return true;
        } else if (endInputMessage != null){
        	return buffer.toString().trim().endsWith(endInputMessage);
        }
        return false;
	}

}
